@@ -1,6 +1,6 @@
 package Devel::PatchPerl;
 BEGIN {
-  $Devel::PatchPerl::VERSION = '0.20';
+  $Devel::PatchPerl::VERSION = '0.22';
 }
 
 # ABSTRACT: Patch perl source a la Devel::PPort's buildperl.pl
@@ -111,6 +111,14 @@ my @patch = (
               [ \&_patch_nbsd_hints ],
             ],
   },
+  {
+    perl => [
+              qr/.*/,
+            ],
+    subs => [
+              [ \&_patch_fbsd_hints ],
+            ],
+  },
   {
     perl => [
               qr/^5\.6\.[0-2]$/,
@@ -1827,6 +1835,324 @@ BADGER
   }
 }
 
+sub _patch_fbsd_hints {
+return unless $^O eq 'freebsd';
+chmod 0755, 'hints/freebsd.sh' or die "$!\n";
+open my $fh, '>', 'hints/freebsd.sh' or die "$\n";
+print $fh <<BADGER;
+# Original based on info from
+# Carl M. Fongheiser <cmf\@ins.infonet.net>
+# Date: Thu, 28 Jul 1994 19:17:05 -0500 (CDT)
+#
+# Additional 1.1.5 defines from 
+# Ollivier Robert <Ollivier.Robert\@keltia.frmug.fr.net>
+# Date: Wed, 28 Sep 1994 00:37:46 +0100 (MET)
+#
+# Additional 2.* defines from
+# Ollivier Robert <Ollivier.Roberti\@keltia.frmug.fr.net>
+# Date: Sat, 8 Apr 1995 20:53:41 +0200 (MET DST)
+#
+# Additional 2.0.5 and 2.1 defined from
+# Ollivier Robert <Ollivier.Robert\@keltia.frmug.fr.net>
+# Date: Fri, 12 May 1995 14:30:38 +0200 (MET DST)
+#
+# Additional 2.2 defines from
+# Mark Murray <mark\@grondar.za>
+# Date: Wed, 6 Nov 1996 09:44:58 +0200 (MET)
+#
+# Modified to ensure we replace -lc with -lc_r, and
+# to put in place-holders for various specific hints.
+# Andy Dougherty <doughera\@lafayette.edu>
+# Date: Tue Mar 10 16:07:00 EST 1998
+#
+# Support for FreeBSD/ELF
+# Ollivier Robert <roberto\@keltia.freenix.fr>
+# Date: Wed Sep  2 16:22:12 CEST 1998
+#
+# The two flags "-fpic -DPIC" are used to indicate a
+# will-be-shared object.  Configure will guess the -fpic, (and the
+# -DPIC is not used by perl proper) but the full define is included to 
+# be consistent with the FreeBSD general shared libs building process.
+#
+# setreuid and friends are inherently broken in all versions of FreeBSD
+# before 2.1-current (before approx date 4/15/95). It is fixed in 2.0.5
+# and what-will-be-2.1
+#
+
+case "\$osvers" in
+0.*|1.0*)
+	usedl="\$undef"
+	;;
+1.1*)
+	malloctype='void *'
+	groupstype='int'
+	d_setregid='undef'
+	d_setreuid='undef'
+	d_setrgid='undef'
+	d_setruid='undef'
+	;;
+2.0-release*)
+	d_setregid='undef'
+	d_setreuid='undef'
+	d_setrgid='undef'
+	d_setruid='undef'
+	;;
+#
+# Trying to cover 2.0.5, 2.1-current and future 2.1/2.2
+# It does not covert all 2.1-current versions as the output of uname
+# changed a few times.
+#
+# Even though seteuid/setegid are available, they've been turned off
+# because perl isn't coded with saved set[ug]id variables in mind.
+# In addition, a small patch is required to suidperl to avoid a security
+# problem with FreeBSD.
+#
+2.0.5*|2.0-built*|2.1*)
+ 	usevfork='true'
+	case "\$usemymalloc" in
+	    "") usemymalloc='n'
+	        ;;
+	esac
+	d_setregid='define'
+	d_setreuid='define'
+	d_setegid='undef'
+	d_seteuid='undef'
+	test -r ./broken-db.msg && . ./broken-db.msg
+	;;
+#
+# 2.2 and above have phkmalloc(3).
+# don't use -lmalloc (maybe there's an old one from 1.1.5.1 floating around)
+2.2*)
+ 	usevfork='true'
+	case "\$usemymalloc" in
+	    "") usemymalloc='n'
+	        ;;
+	esac
+	libswanted=`echo \$libswanted | sed 's/ malloc / /'`
+	libswanted=`echo \$libswanted | sed 's/ bind / /'`
+	# iconv gone in Perl 5.8.1, but if someone compiles 5.8.0 or earlier.
+	libswanted=`echo \$libswanted | sed 's/ iconv / /'`
+	d_setregid='define'
+	d_setreuid='define'
+	d_setegid='define'
+	d_seteuid='define'
+	# d_dosuid='define' # Obsolete.
+	;;
+*)	usevfork='true'
+	case "\$usemymalloc" in
+	    "") usemymalloc='n'
+	        ;;
+	esac
+	libswanted=`echo \$libswanted | sed 's/ malloc / /'`
+	;;
+esac
+
+# Dynamic Loading flags have not changed much, so they are separated
+# out here to avoid duplicating them everywhere.
+case "\$osvers" in
+0.*|1.0*) ;;
+
+1*|2*)	cccdlflags='-DPIC -fpic'
+	lddlflags="-Bshareable \$lddlflags"
+	;;
+
+3*|4*|5*|6*)
+        objformat=`/usr/bin/objformat`
+        if [ x\$objformat = xaout ]; then
+            if [ -e /usr/lib/aout ]; then
+                libpth="/usr/lib/aout /usr/local/lib /usr/lib"
+                glibpth="/usr/lib/aout /usr/local/lib /usr/lib"
+            fi
+            lddlflags='-Bshareable'
+        else
+            libpth="/usr/lib /usr/local/lib"
+            glibpth="/usr/lib /usr/local/lib"
+            ldflags="-Wl,-E "
+            lddlflags="-shared "
+        fi
+        cccdlflags='-DPIC -fPIC'
+        ;;
+*)
+       libpth="/usr/lib /usr/local/lib"
+       glibpth="/usr/lib /usr/local/lib"
+       ldflags="-Wl,-E "
+        lddlflags="-shared "
+        cccdlflags='-DPIC -fPIC'
+       ;;
+esac
+
+case "\$osvers" in
+0*|1*|2*|3*) ;;
+
+*)
+	ccflags="\${ccflags} -DHAS_FPSETMASK -DHAS_FLOATINGPOINT_H"
+	if /usr/bin/file -L /usr/lib/libc.so | /usr/bin/grep -vq "not stripped" ; then
+	    usenm=false
+	fi
+        ;;
+esac
+
+cat <<'EOM' >&4
+
+Some users have reported that Configure halts when testing for
+the O_NONBLOCK symbol with a syntax error.  This is apparently a
+sh error.  Rerunning Configure with ksh apparently fixes the
+problem.  Try
+	ksh Configure [your options]
+
+EOM
+
+# From: Anton Berezin <tobez\@plab.ku.dk>
+# To: perl5-porters\@perl.org
+# Subject: [PATCH 5.005_54] Configure - hints/freebsd.sh signal handler type
+# Date: 30 Nov 1998 19:46:24 +0100
+# Message-ID: <864srhhvcv.fsf\@lion.plab.ku.dk>
+
+signal_t='void'
+d_voidsig='define'
+
+# set libperl.so.X.X for 2.2.X
+case "\$osvers" in
+2.2*)
+    # unfortunately this code gets executed before
+    # the equivalent in the main Configure so we copy a little
+    # from Configure XXX Configure should be fixed.
+    if \$test -r \$src/patchlevel.h;then
+       patchlevel=`awk '/define[ 	]+PERL_VERSION/ {print \$3}' \$src/patchlevel.h`
+       subversion=`awk '/define[ 	]+PERL_SUBVERSION/ {print \$3}' \$src/patchlevel.h`
+    else
+       patchlevel=0
+       subversion=0
+    fi
+    libperl="libperl.so.\$patchlevel.\$subversion"
+    unset patchlevel
+    unset subversion
+    ;;
+esac
+
+# This script UU/usethreads.cbu will get 'called-back' by Configure 
+# after it has prompted the user for whether to use threads.
+cat > UU/usethreads.cbu <<'EOCBU'
+case "\$usethreads" in
+\$define|true|[yY]*)
+        lc_r=`/sbin/ldconfig -r|grep ':-lc_r'|awk '{print \$NF}'|sed -n '\$p'`
+        case "\$osvers" in  
+	0*|1*|2.0*|2.1*)   cat <<EOM >&4
+I did not know that FreeBSD \$osvers supports POSIX threads.
+
+Feel free to tell perlbug\@perl.org otherwise.
+EOM
+	      exit 1
+	      ;;
+
+        2.2.[0-7]*)
+              cat <<EOM >&4
+POSIX threads are not supported well by FreeBSD \$osvers.
+
+Please consider upgrading to at least FreeBSD 2.2.8,
+or preferably to the most recent -RELEASE or -STABLE
+version (see http://www.freebsd.org/releases/).
+
+(While 2.2.7 does have pthreads, it has some problems
+ with the combination of threads and pipes and therefore
+ many Perl tests will either hang or fail.)
+EOM
+	      exit 1
+	      ;;
+
+	[3-5].*)
+	      if [ ! -r "\$lc_r" ]; then
+	      cat <<EOM >&4
+POSIX threads should be supported by FreeBSD \$osvers --
+but your system is missing the shared libc_r.
+(/sbin/ldconfig -r doesn't find any).
+
+Consider using the latest STABLE release.
+EOM
+		 exit 1
+	      fi
+	      # 500016 is the first osreldate in which one could
+	      # just link against libc_r without disposing of libc
+	      # at the same time.  500016 ... up to whatever it was
+	      # on the 31st of August 2003 can still be used with -pthread,
+	      # but it is not necessary.
+
+	      # Anton Berezin says that post 500something we're wrong to be
+	      # to be using -lc_r, and should just be using -pthread on the
+	      # linker line.
+	      # So presumably really we should be checking that \$osver is 5.*)
+	      # and that `/sbin/sysctl -n kern.osreldate` -ge 500016
+	      # or -lt 500something and only in that range not doing this:
+	      ldflags="-pthread \$ldflags"
+
+	      # Both in 4.x and 5.x gethostbyaddr_r exists but
+	      # it is "Temporary function, not threadsafe"...
+	      # Presumably earlier it didn't even exist.
+	      d_gethostbyaddr_r="undef"
+	      d_gethostbyaddr_r_proto="0"
+	      ;;
+
+	*)
+	      # 7.x doesn't install libc_r by default, and Configure
+	      # would fail in the code following
+	      #
+	      # gethostbyaddr_r() appears to have been implemented in 6.x+
+	      ldflags="-pthread \$ldflags"
+	      ;;
+
+	esac
+
+        case "\$osvers" in
+        [1-4]*)
+	    set `echo X "\$libswanted "| sed -e 's/ c / c_r /'`
+	    shift
+	    libswanted="\$*"
+	    ;;
+        *)
+	    set `echo X "\$libswanted "| sed -e 's/ c //'`
+	    shift
+	    libswanted="\$*"
+	    ;;
+	esac
+	    
+	# Configure will probably pick the wrong libc to use for nm scan.
+	# The safest quick-fix is just to not use nm at all...
+	usenm=false
+
+        case "\$osvers" in
+        2.2.8*)
+            # ... but this does not apply for 2.2.8 - we know it's safe
+            libc="\$lc_r"
+            usenm=true
+           ;;
+        esac
+
+        unset lc_r
+
+	# Even with the malloc mutexes the Perl malloc does not
+	# seem to be threadsafe in FreeBSD?
+	case "\$usemymalloc" in
+	'') usemymalloc=n ;;
+	esac
+esac
+EOCBU
+
+# malloc wrap works
+case "\$usemallocwrap" in
+'') usemallocwrap='define' ;;
+esac
+
+# XXX Under FreeBSD 6.0 (and probably most other similar versions)
+# Perl_die(NULL) generates a warning:
+#    pp_sys.c:491: warning: null format string
+# Configure supposedely tests for this, but apparently the test doesn't
+# work.  Volunteers with FreeBSD are needed to improving the Configure test.
+# Meanwhile, the following workaround should be safe on all versions
+# of FreeBSD.
+d_printf_format_null='undef'
+BADGER
+close $fh;
+}
 
 qq[patchin'];
 
@@ -1841,7 +2167,7 @@ Devel::PatchPerl - Patch perl source a la Devel::PPort's buildperl.pl
 
 =head1 VERSION
 
-version 0.20
+version 0.22
 
 =head1 SYNOPSIS
 
@@ -1854,10 +2180,10 @@ version 0.20
 
 =head1 DESCRIPTION
 
-Devel::PatchPerl is a modularisation of the patching code contained in L<Devel::PPort>'s 
+Devel::PatchPerl is a modularisation of the patching code contained in L<Devel::PPort>'s
 C<buildperl.pl>.
 
-It does not build perls, it merely provides an interface to the source patching 
+It does not build perls, it merely provides an interface to the source patching
 functionality.
 
 =head1 FUNCTION
@@ -1884,7 +2210,7 @@ Chris Williams <chris@bingosnet.co.uk>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2010 by Chris Williams and Marcus Holland-Moritz.
+This software is copyright (c) 2011 by Chris Williams and Marcus Holland-Moritz.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
